<?php

declare(strict_types=1);

namespace Drupal\Tests\confirmation\Kernel;

use Drupal\confirmation\Entity\Event\ConfirmationEvent;
use Drupal\confirmation_event_test\LoggingConfirmationEventSubscriber;
use Drupal\confirmation_example\Entity\ExampleConfirmationBundle;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;

/**
 * Kernel tests with more dependencies.
 *
 * @group confirmation
 */
final class ConfirmationKernelTest extends KernelTestBase {

  protected static $modules = [
    'node',
    'user',
    'field',
    'confirmation',
    'confirmation_example',
    'confirmation_event_test',
  ];

  protected NodeInterface $node;

  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installSchema('node', 'node_access');
    $this->installEntitySchema('node');
    $this->installEntitySchema('confirmation');

    $this->node = Node::create([
      'type' => 'article',
      'title' => 'ArticleToConfirm',
    ]);
    $this->node->save();
  }

  public function testConfirmationBundles() {
    /** @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo */
    $entityTypeBundleInfo = \Drupal::service('entity_type.bundle.info');
    $bundleInfo = $entityTypeBundleInfo->getBundleInfo('confirmation');
    $this->assertSame(['confirmation_example'], array_keys($bundleInfo));
    $this->assertSame(ExampleConfirmationBundle::class, $bundleInfo['confirmation_example']['class'] ?? NULL);
  }

  public function testConfirmationBundleFields() {
    /** @var \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager */
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $fieldStorageDefinitions = $entityFieldManager->getFieldStorageDefinitions('confirmation');
    $this->assertArrayHasKey('confirmation_example__node', $fieldStorageDefinitions);
  }

  public function testConfirmationGetNode() {
    $confirmation = ExampleConfirmationBundle::create()
      ->setNode($this->node);
    $confirmation->save();
    /** @var ExampleConfirmationBundle $reloaded */
    $reloaded = \Drupal::entityTypeManager()->getStorage('confirmation')
      ->loadUnchanged($confirmation->id());
    $this->assertInstanceOf(NodeInterface::class, $reloaded->getNode());
  }

  public function testConfirmationTokens() {
    \Drupal::moduleHandler()->loadInclude('confirmation', 'inc', 'confirmation.tokens');

    $confirmation = ExampleConfirmationBundle::create()
      ->setEmail('noam.chomski@example.com');
    $confirmation->save();
    $template = 'Confirm [confirmation:email] at [confirmation:response-url] until [confirmation:expiration]';
    $result = \Drupal::token()->replacePlain($template, ['confirmation' => $confirmation]);
    // @see \PHPUnit\Framework\Constraint\StringMatchesFormatDescription::createPatternFromFormat
    $this->assertStringMatchesFormat('Confirm noam.chomski@example.com at http://%s/confirmation/1/%x until %s', $result);
  }

  public function testDefaultExpiration() {
    $future1 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node);
    $this->assertSame(ExampleConfirmationBundle::getExpirationInterval(), $future1->getExpirationTime() - \Drupal::time()->getRequestTime());
  }

  public function testPurge() {
    $future1 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node);
    $future1->save();

    $future2 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node);
    $future2->save();

    $past1 = ExampleConfirmationBundle::create(['bundle' => 'confirmation_example',])
      ->setNode($this->node)
      ->setExpirationInterval(-1);
    $past1->save();

    $this->assertSame(TRUE, $past1->isExpired());
    $this->assertSame(FALSE, $future1->isExpired());

    $toPurge = ExampleConfirmationBundle::getEntityIdsToPurge('confirmation_example', 100);
    self::assertSame([$past1->id()], array_values($toPurge));
  }

  public function testEvents(): void {
    $loggingEventSubscriber = \Drupal::service('confirmation_event_test.subscriber');
    assert($loggingEventSubscriber instanceof LoggingConfirmationEventSubscriber);
    $loggingEventSubscriber->getAndClearEvents();
    $confirmation = ExampleConfirmationBundle::create();
    $confirmation->setNode($this->node);
    $confirmation->save();
    $confirmation->save();
    $confirmation->setState(TRUE);
    $this->assertEmpty($loggingEventSubscriber->getAndClearEvents());
    $confirmation->save();
    $events = $loggingEventSubscriber->getAndClearEvents();
    $this->assertCount(2, $events);
    $this->assertSame('confirmation.confirmation_example.state_settled', $events[0][1]);
    $this->assertSame('confirmation.state_settled', $events[1][1]);
    $this->assertInstanceOf(ConfirmationEvent::class, $events[0][0]);
    $this->assertInstanceOf(ConfirmationEvent::class, $events[1][0]);
  }

}
